///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; 
using ILNumerics.Drawing; 
using ILNumerics.Drawing.Interfaces; 

namespace ILNumerics.Drawing.Graphs {
    /// <summary>
    /// class rendering plain color-coded area graphs
    /// </summary>
    /// <remarks>This class derives from ILFilledGraph and adds a ZPosition property.</remarks>
    public abstract class ILImageSCGraph : ILFilledGraph, IILLegendRenderer, IILPanelConfigurator {

        #region attributes 
        protected float m_zPosition;
        #endregion

        #region Properties
        /// <summary>
        /// Z position for ImageSC graph (default: -Inf)
        /// </summary>
        /// <remarks>The position of the ImageSC plane is specified by a Z coordinate.
        /// The coordinate can set to the following values: 
        /// <list type="bullet">
        /// <item>any single precision value, will position the plane on that specific value</item>
        /// <item><i>negative infinity</i>, will position the plane on the bottom of the viewing cube</item>
        /// <item><i>positive infinity</i>, position the plane on top of the unit cube.</item></list>
        /// </remarks>
        public float ZPosition {
            get {
                return m_zPosition; 
            }
            set {
                m_zPosition = value; 
                OnChanged("ZPosition"); 
            }
        }
        #endregion

        #region constructor
        public ILImageSCGraph(ILPanel panel, ILArray<float> X, ILArray<float> Y,
                                ILArray<float> Z, ILArray<float> C,
                                ILClippingData clippingContainer)
            : base(panel, X, Y, Z, C, clippingContainer) {
            using (ILScope.Enter(X, Y, Z, C)) {
                m_cols = m_sourceArray.Size[1] + 1;
                m_rows = m_sourceArray.Size[0] + 1;
                m_Vertcount = m_rows * m_cols;
                // defaults
                m_zPosition = float.NegativeInfinity;
            }
        }
        #endregion

        public override bool Is3DGraph() {
            return true;
        }

        #region IILPanelConfigurator
        public void ConfigurePanel(ILPanel panel) {
            panel.InteractiveMode = InteractiveModes.ZoomRectangle;
            panel.AspectRatio = AspectRatioMode.StretchToFill;
            panel.PlotBoxScreenSizeMode = PlotBoxScreenSizeMode.StrictOptimal;
            panel.ClipViewData = false;
            panel.DefaultView.Set(0f, 0f, panel.DefaultView.Distance);
        }
        #endregion

        #region IILRenderToLegend
        public abstract void DrawToLegend(ILRenderProperties p, Rectangle sampleRect, Rectangle labelRect);
        public Size LabelSize { get { return m_label.Size; } }
        #endregion

    }
}
